#Developer: Curly60e
#Tester: __B__T__C__
#ℙ𝕪𝔹𝕃𝕆ℂ𝕂 𝕚𝕥𝕤 𝕒 𝔹𝕚𝕥𝕔𝕠𝕚𝕟 𝔻𝕒𝕤𝕙𝕓𝕠𝕒𝕣𝕕 𝕨𝕚𝕥𝕙 ℂ𝕪𝕡𝕙𝕖𝕣𝕡𝕦𝕟𝕜 𝕒𝕖𝕤𝕥𝕙𝕖𝕥𝕚𝕔.


import base64, codecs, json, requests
import pickle
import os
import os.path
import qrcode
import sys
import simplejson as json
import time as t
import numpy as np
from cfonts import render, say
from art import *
from pblogo import *
from PIL import Image
from robohash import Robohash


lndconnectload = {"ip_port":"", "tls":"", "macaroon":"", "ln":""}
settingsClock = {"gradient":"", "design":"", "colorA":"", "colorB":""}


def clear(): # clear the screen
    os.system('cls' if os.name=='nt' else 'clear')
def closed():
    print("<<< Back Control + C.\n\n")

#-------------------------RPC BITCOIN NODE CONNECTION

def rpc(method, params=[]):
    payload = json.dumps({
        "jsonrpc": "2.0",
        "id": "minebet",
        "method": method,
        "params": params
    })
    path = {"ip_port":"", "rpcuser":"", "rpcpass":"", "bitcoincli":""}
    if os.path.isfile('bclock.conf'): # Check if the file 'bclock.conf' is in the same folder
        pathv = pickle.load(open("bclock.conf", "rb")) # Load the file 'bclock.conf'
        path = pathv # Copy the variable pathv to 'path'
    return requests.post(path['ip_port'], auth=(path['rpcuser'], path['rpcpass']), data=payload).json()['result']

def remoteHalving():

    b = rpc('getblockcount')
    c = str(b)
    oneh = 0 - int(c) + 210000
    twoh = 210000 - int(c) + 210000
    thrh = 420000 - int(c) + 210000
    forh = 630000 - int(c) + 210000
    fifh = 840000 - int(c) + 210000
    sixh = 1050000 - int(c) + 210000
    sevh = 1260000 - int(c) + 210000
    eith = 1470000 - int(c) + 210000
    ninh = 1680000 - int(c) + 210000
    tenh = 1890000 - int(c) + 210000

    q = """
    \033[0;37;40m------------------- HALVING HISTORY -------------------

            1st  Halving: in {} Blocks {}
            2nd  Halving: in {} Blocks {}
            3rd  Halving: in {} Blocks {}
            4th  Halving: in {} Blocks {}
            5th  Halving: in {} Blocks {}
            6th  Halving: in {} Blocks {}
            7th  Halving: in {} Blocks {}
            8th  Halving: in {} Blocks {}
            9th  Halving: in {} Blocks {}
            10th Halving: in {} Blocks {}

    -------------------------------------------------------
    """.format("0" if int(c) == 210000 else oneh,"\033[1;32;40mCOMPLETE\033[0;37;40m","0" if int(c) == 420000 else twoh,"\033[1;32;40mCOMPLETE\033[0;37;40m", "0" if int(c) == 630000 else thrh,"\033[1;32;40mCOMPLETE\033[0;37;40m","0" if int(c) == 840000 else forh,"\033[1;32;40mCOMPLETE\033[0;37;40m" if int(c) >= 840000 else "\033[1;35;40mPENDING\033[0;37;40m", "0" if int(c) >= 1050000 else fifh , "\033[1;32;40mCOMPLETE\033[0;37;40m" if int(c) >= 1050000 else "\033[1;35;40mPENDING\033[0;37;40m", sixh, "\033[1;32;40mCOMPLETE\033[0;37;40m" if int(c) >= 1260000 else "\033[1;35;40mPENDING\033[0;37;40m", sevh,"\033[1;32;40mCOMPLETE\033[0;37;40m" if int(c) >= 1470000 else "\033[1;35;40mPENDING\033[0;37;40m", eith,"\033[1;32;40mCOMPLETE\033[0;37;40m" if int(c) >= 1680000 else "\033[1;35;40mPENDING\033[0;37;40m", ninh, "\033[1;32;40mCOMPLETE\033[0;37;40m" if int(c) >= 1890000 else "\033[1;35;40mPENDING\033[0;37;40m", tenh, "\033[1;32;40mCOMPLETE\033[0;37;40m" if int(c) >= 1890000 else "\033[1;35;40mPENDING\033[0;37;40m")
    print(q)
    input("\nContinue...")


def remotegetblock():
    if os.path.isfile('pyblocksettingsClock.conf') or os.path.isfile('pyblocksettingsClock.conf'): # Check if the file 'bclock.conf' is in the same folder
        settingsv = pickle.load(open("pyblocksettingsClock.conf", "rb")) # Load the file 'bclock.conf'
        settingsClock = settingsv # Copy the variable pathv to 'path'
    else:
        settingsClock = {"gradient":"", "design":"block", "colorA":"green", "colorB":"yellow"}
        pickle.dump(settingsClock, open("pyblocksettingsClock.conf", "wb"))
    b = rpc('getblockcount')
    c = str(b)
    a = c
    output = render(str(c), colors=[settingsClock['colorA'], settingsClock['colorB']], align='center')
    print("\x1b[?25l" + output)
    while True:
        x = a
        b = rpc('getblockcount')
        c = str(b)
        if c > a:
            clear()
            closed()
            output = render(str(c), colors=[settingsClock['colorA'], settingsClock['colorB']], align='center')
            print("\a\x1b[?25l" + output)
            a = c

def remotegetblockcount(): # get access to bitcoin-cli with the command getblockcount
    while True:
        try:
            a = rpc('getblockchaininfo')
            d = a
            clear()
            blogo()
            closed()
            print("""
            ----------------------------------------------------------------------------
            \tGET BLOCKCHAIN INFORMATION
            Chain: {}
            Blocks: {}
            Best BlockHash: {}
            Difficulty: {}
            Verification Progress: {}
            Size on Disk: {}
            Pruned: {}
            ----------------------------------------------------------------------------
            """.format(d['chain'], d['blocks'], d['bestblockhash'], d['difficulty'], d['verificationprogress'], d['size_on_disk'], d['pruned']))
            t.sleep(2)
        except:
            break

def remoteconsole(): # get into the console from bitcoin-cli
    print("\t\033[0;37;40mThis is \033[1;33;40mBitcoin-cli's \033[0;37;40mconsole. Type your respective commands you want to display.\n\n")
    while True:
        cle = input("\033[1;32;40mconsole $>: \033[0;37;40m")
        a = rpc(cle)
        print(a)

def runthenumbersConn():
    try:
        b = rpc('gettxoutsetinfo')
        c = str(b)
        print(c)
        input("\nContinue...")
    except:
        pass

#-------------------------END RPC BITCOIN NODE CONNECTION

def consoleLN(): # get into the console from bitcoin-cli
    lndconnectData= pickle.load(open("config/blndconnect.conf", "rb")) # Load the file 'bclock.conf'
    lndconnectload = lndconnectData # Copy the variable pathv to 'path'
    print("\t\033[0;37;40mThis is \033[1;33;40mLncli's \033[0;37;40mconsole. Type your respective commands you want to display.\n\n")
    while True:
        cle = input("\033[1;32;40mconsole $>: \033[0;37;40m")
        lsd = os.popen(lndconnectload['ln'] + " " + cle)
        lsd0 = lsd.read()
        lsd1 = str(lsd0)
        print(lsd1)
        lsd.close()

def locallistpeersQQ():
    lndconnectData= pickle.load(open("config/blndconnect.conf", "rb")) # Load the file 'bclock.conf'
    lndconnectload = lndconnectData # Copy the variable pathv to 'path'
    qr = qrcode.QRCode(
    version=1,
    error_correction=qrcode.constants.ERROR_CORRECT_L,
    box_size=10,
    border=4,
    )
    lncli = " listpeers"
    while True:
        clear()
        print("\033[1;32;40m")
        blogo()
        print("\033[0;37;40m")
        print("<<< Back to the Main Menu Press Control + C.\n\n")
        lsd = os.popen(lndconnectload['ln'] + lncli).read()
        lsd0 = str(lsd)
        d = json.loads(lsd0)
        n = d['peers']
        try:
            print("\n\tLIST PEERS\n")
            for item_ in n:
                s = item_
                hash = s['pub_key']
                rh = Robohash(hash)
                rh.assemble(roboset='set1')
                if not os.path.isfile(str(f'{hash}.png')):
                    with open(f'{hash}.png', "wb") as f:
                        rh.img.save(f, format="png")

                    img_path = open(f'{hash}.png', "rb")
                    img = Image.open(img_path)

                    h = 1
                    w = int((img.width / img.height) * 5)

                    img = img.resize((w,h), Image.ANTIALIAS)
                    img_arr = np.asarray(img)
                    h,w,c = img_arr.shape

                img_path = open(f'{hash}.png', "rb")
                img = Image.open(img_path)

                h = 1
                w = int((img.width / img.height) * 5)

                img = img.resize((w,h), Image.ANTIALIAS)
                img_arr = np.asarray(img)
                h,w,c = img_arr.shape

                for x in range(h):
                    for y in range(w):
                        pix = img_arr[x][y]
                        print(get_color(pix[0], pix[1], pix[2]), sep='', end='')
                    print()
                print("PubKey: " + s['pub_key'] + " @" + s['address'])

            nd = input("\nSelect PubKey: ")
            for item in n:
                s = item
                nn = s['pub_key']
                if nd == nn:
                    hash = s['pub_key']
                    rh = Robohash(hash)
                    rh.assemble(roboset='set1')

                    img_path = open(f'{hash}.png', "rb")
                    img = Image.open(img_path)

                    h = 20
                    w = int((img.width / img.height) * 50)

                    img = img.resize((w,h), Image.ANTIALIAS)
                    img_arr = np.asarray(img)
                    h,w,c = img_arr.shape

                    for x in range(h):
                        for y in range(w):
                            pix = img_arr[x][y]
                            print(get_color(pix[0], pix[1], pix[2]), sep='', end='')
                        print()
                    print("\n----------------------------------------------------------------------------------------------------")
                    print("""
                        PEER DECODED\n
                        Bytes Sent: {}
                        Bytes Recv: {}
                        Sat Sent: {} sats
                        Sat Recv: {} sats
                    """.format(s['bytes_sent'], s['bytes_recv'], s['sat_sent'], s['sat_recv']))
                    print("-----------------------------------------------------------------------------------------------------\n")
                    print("\n\tPeer: " + nd)
                    print("\033[1;30;47m")
                    qr.add_data(s['pub_key'])
                    qr.print_ascii()
                    print("\033[0;37;40m")
                    qr.clear()

            pp = input("\nDo you want to disconnect? Y/n: ")
            if pp in ["Y", "y"]:
                lsd = os.popen(lndconnectload['ln'] + " disconnect" + " " + nd).read()
                lsd0 = str(lsd)
                d = json.loads(lsd0)
                print("\n\tDisconnected from peer " + nd)
                input("\nContinue... ")
            elif pp in ["N", "n"]:
                input("\nContinue... ")
        except:
            break

def localconnectpeer():
    lndconnectData= pickle.load(open("config/blndconnect.conf", "rb")) # Load the file 'bclock.conf'
    lndconnectload = lndconnectData # Copy the variable pathv to 'path'
    try:
        clear()
        print("\033[1;32;40m")
        blogo()
        print("\033[0;37;40m")
        print("<<< Back to the Main Menu Press Control + C.\n\n")
        print("\n\tCONNECT TO NEW PEER\n")
        a = input("Insert PeerID@IP:PORT: ")
        lncli = " connect "
        lsd = os.popen(lndconnectload['ln'] + lncli + a).read()
        lsd0 = str(lsd)
        print(lsd0)
        input("\nContinue... ")
    except:
        pass

def locallistchaintxns():
    lndconnectData= pickle.load(open("config/blndconnect.conf", "rb")) # Load the file 'bclock.conf'
    lndconnectload = lndconnectData # Copy the variable pathv to 'path'
    qr = qrcode.QRCode(
    version=1,
    error_correction=qrcode.constants.ERROR_CORRECT_L,
    box_size=10,
    border=4,
    )
    lncli = " listchaintxns"
    lsd = os.popen(lndconnectload['ln'] + lncli).read()
    lsd0 = str(lsd)
    d = json.loads(lsd0)
    n = d['transactions']
    while True:
        clear()
        print("\033[1;32;40m")
        blogo()
        print("\033[0;37;40m")
        print("<<< Back to the Main Menu Press Control + C.\n\n")
        print("\t\nTransactions\n")
        try:
            print("\n\tLIST ONCHAIN TRANSACTIONS\n")
            for item_ in n:
                s = item_

                print("Transaction Hash: " + s['tx_hash'] + " " + s['amount'] + " sats")
            nd = input("\nSelect RHash: ")

            for item in n:
                s = item
                nn = s['tx_hash']
                trx = s['dest_addresses']
                if nd == nn:
                    print("\n----------------------------------------------------------------------------------------------------")
                    print("""
                    \nONCHAIN TRANSACTION DECODED
                    Amount: {} sats
                    Tx Hash: {}
                    Block Hash: {}
                    Block Height: {}
                    Confirmations: {}
                    Destination: {}
                    """.format(s['amount'], s['tx_hash'], s['block_hash'], s['block_height'], s['num_confirmations'], trx))
                    print("-----------------------------------------------------------------------------------------------------\n")
                    print("\nTransaction Hash")
                    print("\033[1;30;47m")
                    qr.add_data(s['tx_hash'])
                    qr.print_ascii()
                    print("\033[0;37;40m")
                    qr.clear()
            input("\nContinue... ")
        except:
            break

def locallistinvoices():
    lndconnectData= pickle.load(open("config/blndconnect.conf", "rb")) # Load the file 'bclock.conf'
    lndconnectload = lndconnectData # Copy the variable pathv to 'path'
    qr = qrcode.QRCode(
    version=1,
    error_correction=qrcode.constants.ERROR_CORRECT_L,
    box_size=10,
    border=4,
    )
    lncli = " listinvoices"
    lsd = os.popen(lndconnectload['ln'] + lncli).read()
    lsd0 = str(lsd)
    d = json.loads(lsd0)
    n = d['invoices']
    while True:
        clear()
        print("\033[1;32;40m")
        blogo()
        print("\033[0;37;40m")
        print("<<< Back to the Main Menu Press Control + C.\n\n")
        print("\tInvoices\n")
        try:
            print("\n\tLIST INVOICES\n")
            for item_ in n:
                s = item_

                print("Invoice: " + s['r_hash'] + " " + s['state'])

            nd = input("\nSelect RHash: ")

            for item in n:
                s = item
                nn = s['r_hash']
                if nd == nn:
                    print("\n----------------------------------------------------------------------------------------------------")
                    print("""
                    \nINVOICE DECODED
                    Memo: {}
                    Invoice: {}
                    Amount: {} sats
                    State: {}
                    """.format(s['memo'], s['payment_request'], s['amt_paid_sat'], s['state']))
                    print("----------------------------------------------------------------------------------------------------\n")
                    print("\033[1;30;47m")
                    qr.add_data(s['payment_request'])
                    qr.print_ascii()
                    print("\033[0;37;40m")
                    qr.clear()
            input("\nContinue... ")
        except:
            break

def locallistchannels():
    lndconnectData= pickle.load(open("config/blndconnect.conf", "rb")) # Load the file 'bclock.conf'
    lndconnectload = lndconnectData # Copy the variable pathv to 'path'
    lncli = " listchannels"
    lsd = os.popen(lndconnectload['ln'] + lncli).read()
    lsd0 = str(lsd)
    d = json.loads(lsd0)
    n = d['channels']
    while True:
        clear()
        print("\033[1;32;40m")
        blogo()
        print("\033[0;37;40m")
        print("<<< Back to the Main Menu Press Control + C.\n\n")
        print("\t\nChannels\n")
        try:
            print("\n\tLIST CHANNELS\n")
            for item_ in n:
                s = item_
                hash = s['remote_pubkey']
                rh = Robohash(hash)
                rh.assemble(roboset='set1')
                if not os.path.isfile(str(f'{hash}.png')):
                    with open(f'{hash}.png', "wb") as f:
                        rh.img.save(f, format="png")

                    img_path = open(f'{hash}.png', "rb")
                    img = Image.open(img_path)

                    h = 1
                    w = int((img.width / img.height) * 5)

                    img = img.resize((w,h), Image.ANTIALIAS)
                    img_arr = np.asarray(img)
                    h,w,c = img_arr.shape

                img_path = open(f'{hash}.png', "rb")
                img = Image.open(img_path)

                h = 1
                w = int((img.width / img.height) * 5)

                img = img.resize((w,h), Image.ANTIALIAS)
                img_arr = np.asarray(img)
                h,w,c = img_arr.shape

                for x in range(h):
                    for y in range(w):
                        pix = img_arr[x][y]
                        print(get_color(pix[0], pix[1], pix[2]), sep='', end='')
                    print()
                print("Node ID: " + s['remote_pubkey'])

            nd = input("\nSelect a Node ID: ")
            for item in n:
                s = item
                nn = s['remote_pubkey']
                if nd == nn:
                    hash = s['remote_pubkey']
                    rh = Robohash(hash)
                    rh.assemble(roboset='set1')

                    img_path = open(f'{hash}.png', "rb")
                    img = Image.open(img_path)

                    h = 20
                    w = int((img.width / img.height) * 50)

                    img = img.resize((w,h), Image.ANTIALIAS)
                    img_arr = np.asarray(img)
                    h,w,c = img_arr.shape

                    for x in range(h):
                        for y in range(w):
                            pix = img_arr[x][y]
                            print(get_color(pix[0], pix[1], pix[2]), sep='', end='')
                        print()
                    print("\n----------------------------------------------------------------------------------------------------")
                    print("""
                    \tCHANNEL DECODED
                    Active: {}
                    Node ID: {}
                    Channel Point: {}
                    Channel Capacity: {} sats
                    Local Balance: {} sats
                    Remote Balance: {} sats
                    Total Sent: {} sats
                    Total Received: {} sats
                    """.format(s['active'], s['remote_pubkey'], s['channel_point'], s['capacity'], s['local_balance'], s['remote_balance'], s['total_satoshis_sent'], s['total_satoshis_received']))
                    print("----------------------------------------------------------------------------------------------------\n")

            input("\nContinue... ")
        except:
            break

def localgetinfo():
    lndconnectData= pickle.load(open("config/blndconnect.conf", "rb")) # Load the file 'bclock.conf'
    lndconnectload = lndconnectData # Copy the variable pathv to 'path'
    qr = qrcode.QRCode(
    version=1,
    error_correction=qrcode.constants.ERROR_CORRECT_L,
    box_size=10,
    border=4,
    )
    lncli = " getinfo"
    lsd = os.popen(lndconnectload['ln'] + lncli).read()
    lsd0 = str(lsd)
    d = json.loads(lsd0)
    hash = d['identity_pubkey']
    rh = Robohash(hash)
    rh.assemble(roboset='set1')
    if not os.path.isfile(str(f'{hash}.png')):
        with open(f'{hash}.png', "wb") as f:
            rh.img.save(f, format="png")

        img_path = open(f'{hash}.png', "rb")
        img = Image.open(img_path)

        h = 20
        w = int((img.width / img.height) * 50)

        img = img.resize((w,h), Image.ANTIALIAS)
        img_arr = np.asarray(img)
        h,w,c = img_arr.shape

    img_path = open(f'{hash}.png', "rb")
    img = Image.open(img_path)

    h = 20
    w = int((img.width / img.height) * 50)

    img = img.resize((w,h), Image.ANTIALIAS)
    img_arr = np.asarray(img)
    h,w,c = img_arr.shape

    for x in range(h):
        for y in range(w):
            pix = img_arr[x][y]
            print(get_color(pix[0], pix[1], pix[2]), sep='', end='')
        print()
    print("\n----------------------------------------------------------------------------------------------------")
    print("""
    \tNODE INFORMATION

    Version: {}
    Node ID: {}
    Alias: {}
    Color: {}
    Pending Channels: {}
    Active Channels: {}
    Inactive Channels: {}
    Peers: {}
    URLS: {}
    """.format(d['version'], d['identity_pubkey'], d['alias'], d['color'], d['num_pending_channels'], d['num_active_channels'], d['num_inactive_channels'], d['num_peers'], d['uris']))
    print("\033[1;30;47m")
    qr.add_data(d['identity_pubkey'])
    qr.print_ascii()
    print("\033[0;37;40m")
    qr.clear()
    print("----------------------------------------------------------------------------------------------------\n")
    input("\nContinue... ")

def localaddinvoice():
    lndconnectData= pickle.load(open("config/blndconnect.conf", "rb")) # Load the file 'bclock.conf'
    lndconnectload = lndconnectData # Copy the variable pathv to 'path'
    lncli = " addinvoice"
    lsd = os.popen(lndconnectload['ln'] + lncli).read()
    lsd0 = str(lsd)
    d = json.loads(lsd0)
    qr = qrcode.QRCode(
    version=1,
    error_correction=qrcode.constants.ERROR_CORRECT_L,
    box_size=10,
    border=4,
    )
    try:
        amount = input("Amount in sats: ")
        mem = input("Memo: ")
        memo = mem.replace(" ","_")
        lsd = os.popen(lndconnectload['ln'] + lncli + " --memo {}-PyBLOCK --amt {}".format(memo, amount)).read()
        lsd0 = str(lsd)
        d = json.loads(lsd0)
        print("\033[1;30;47m")
        qr.add_data(d['payment_request'])
        qr.print_ascii()
        print("\033[0;37;40m")
        qr.clear()
        print("Lightning Invoice: " + d['payment_request'])
        b = str(d['payment_request'])
        while True:
            lsd = os.popen(lndconnectload['ln'] + " decodepayreq " + b).read()
            lsd0 = str(lsd)
            d = json.loads(lsd0)
            r = d['payment_hash']
            lsdn = os.popen(lndconnectload['ln'] + " lookupinvoice " + r).read()
            lsdn0 = str(lsdn)
            n = json.loads(lsdn0)
            if n['state'] == 'SETTLED':
                print("\033[1;32;40m")
                clear()
                blogo()
                tick()
                print("\033[0;37;40m")
                t.sleep(2)
                break
            elif n['state'] == 'CANCELED':
                print("\033[1;31;40m")
                clear()
                blogo()
                canceled()
                print("\033[0;37;40m")
                t.sleep(2)
                break
    except:
        pass

def localpayinvoice():
    lndconnectData= pickle.load(open("config/blndconnect.conf", "rb")) # Load the file 'bclock.conf'
    lndconnectload = lndconnectData # Copy the variable pathv to 'path'
    try:
        invoiceN = input("Insert the invoice to pay: ")
        invoice = invoiceN.lower()
        lncli = " payinvoice "
        lsd = os.popen(lndconnectload['ln'] + " decodepayreq " + invoice).read()
        lsd0 = str(lsd)
        d = json.loads(lsd0)
        if d['num_satoshis'] == "0":
            amt = " --amt "
            amount =  input("Amount in satoshis: ")
            os.system(lndconnectload['ln'] + lncli + invoice + amt + amount)
        else:
            os.system(lndconnectload['ln'] + lncli + invoice )
        t.sleep(2)
    except:
        pass

def localgetnetworkinfo():
    lndconnectData= pickle.load(open("config/blndconnect.conf", "rb")) # Load the file 'bclock.conf'
    lndconnectload = lndconnectData # Copy the variable pathv to 'path'
    lncli = " getnetworkinfo"
    lsd = os.popen(lndconnectload['ln'] + lncli).read()
    lsd0 = str(lsd)
    d = json.loads(lsd0)
    print("\n----------------------------------------------------------------------------------------------------")
    print("""
    \tLIGHTNING NETWORK INFORMATION
    Numbers of Nodes: {}
    Numbers of Channels: {}
    Total Network Capacity: {} sats
    Average Channel Size: {}
    Minimum Channel Size: {}
    Maximum Channel Size: {}
    Median Channel Size: {} sats
    Zombie channels: {}
    """.format(d['num_nodes'], d['num_channels'], d['total_network_capacity'], d['avg_channel_size'], d['min_channel_size'], d['max_channel_size'], d['median_channel_size_sat'], d['num_zombie_chans']))
    print("----------------------------------------------------------------------------------------------------\n")
    input("\nContinue... ")

def localFullProtocol():
    lndconnectData= pickle.load(open("config/blndconnect.conf", "rb")) # Load the file 'bclock.conf'
    lndconnectload = lndconnectData # Copy the variable pathv to 'path'

    proto1 = """lncli listinvoices | grep "34349334" | tr -d '"' | tr -d ',' | sed 's/34349334/0a0a2d5079424c4f434b204d6573736167652052656365697665643a200a/g' | html2text | xxd -r -p | xargs --null"""
    proto2 = """lncli listinvoices | grep "7629171" | tr -d '"' | tr -d ',' | sed 's/7629171/0a0a2d5079424c4f434b204d6573736167652052656365697665643a200a/g' | html2text | xxd -r -p | xargs --null"""
    proto3 = """lncli listinvoices | grep "34343434" | tr -d '"' | tr -d ',' | sed 's/34343434/0a0a2d5079424c4f434b204d6573736167652052656365697665643a200a/g' | html2text | xxd -r -p | xargs --null"""
    p1 = os.popen(proto1).read()
    p2 = os.popen(proto2).read()
    p3 = os.popen(proto3).read()

    proto1 = """lncli listpayments | grep "34349334" | tr -d '"' | tr -d ',' | sed 's/34349334/0a0a202d5079424c4f434b204d6573736167653a200a/g' | html2text | xxd -r -p | xargs --null"""
    proto2 = """lncli listpayments | grep "7629171" | tr -d '"' | tr -d ',' | sed 's/7629171/0a0a202d5079424c4f434b204d6573736167653a200a/g' | html2text | xxd -r -p | xargs --null"""
    proto3 = """lncli listpayments | grep "34343434" | tr -d '"' | tr -d ',' | sed 's/34343434/0a0a202d5079424c4f434b204d6573736167653a200a/g' | html2text | xxd -r -p | xargs --null"""
    p1 = os.popen(proto1).list()
    p2 = os.popen(proto2).list()
    p3 = os.popen(proto3).list()



def localkeysend():
    lndconnectData= pickle.load(open("config/blndconnect.conf", "rb")) # Load the file 'bclock.conf'
    lndconnectload = lndconnectData # Copy the variable pathv to 'path'
    try:
        closed()
        print("\n\tYou are going to send a payment using KeySend - Note: You don't need any invoice, just your peer ID.\n")
        lncli = " sendpayment "
        node = input("Send to NodeID: ")
        amount = input("Amount in sats: ")
        while True:
            if amount in ["", "0"]:
                amount = input("\nAmount in sats: ")
            else:
                break
        os.system(
            f"""lncli sendpayment --keysend --d={node} --amt={amount}"""
            + """ --final_cltv_delta=40"""
        )

        input("\nContinue...")
    except:
        pass

def localchatsendA():
    lndconnectData= pickle.load(open("config/blndconnect.conf", "rb")) # Load the file 'bclock.conf'
    lndconnectload = lndconnectData # Copy the variable pathv to 'path'
    try:
        closed()
        print("\n\tWrite.\n")
        lncli = " sendpayment "
        node = input("Send to NodeID: ")
        amount = input("Amount in sats: ")
        message = input("Message: ")
        encoded_message = message.encode('utf-8')
        hex_encoded_message = encoded_message.hex()
        input("\nContinue...")

        while True:
            if amount in ["", "0"]:
                amount = input("\nAmount in sats: ")
            else:
                break
        os.system(
            f"""lncli sendpayment --keysend --d={node} --amt={amount}"""
            + """ --data 34349334="""
            + hex_encoded_message
        )

        input("\nContinue...")
    except:
        pass

def localchatnewA():
    lndconnectData= pickle.load(open("config/blndconnect.conf", "rb")) # Load the file 'bclock.conf'
    lndconnectload = lndconnectData # Copy the variable pathv to 'path'
    try:
        closed()
        print("\n\tRead.\n")
        os.system("""lncli listinvoices | grep "34349334" | tr -d '"' | tr -d ',' | sed 's/34349334/0a0a2d5079424c4f434b204d6573736167652052656365697665643a200a/g' | html2text | xxd -r -p | xargs --null""")
        input("\nContinue...")
    except:
        pass

def localchatlistA():
    lndconnectData= pickle.load(open("config/blndconnect.conf", "rb")) # Load the file 'bclock.conf'
    lndconnectload = lndconnectData # Copy the variable pathv to 'path'
    try:
        closed()
        print("\n\tList.\n")
        os.system("""lncli listpayments | grep "34349334" | tr -d '"' | tr -d ',' | sed 's/34349334/0a0a202d5079424c4f434b204d6573736167653a200a/g' | html2text | xxd -r -p | xargs --null""")
        input("\nContinue...")
    except:
        pass

def localchatsendB():
    lndconnectData= pickle.load(open("config/blndconnect.conf", "rb")) # Load the file 'bclock.conf'
    lndconnectload = lndconnectData # Copy the variable pathv to 'path'
    try:
        closed()
        print("\n\tWrite.\n")
        lncli = " sendpayment "
        node = input("Send to NodeID: ")
        amount = input("Amount in sats: ")
        message = input("Message: ")
        encoded_message = message.encode('utf-8')
        hex_encoded_message = encoded_message.hex()
        print(encoded_message.hex())
        input("\nContinue...")

        while True:
            if amount in ["", "0"]:
                amount = input("\nAmount in sats: ")
            else:
                break
        os.system(
            f"""lncli sendpayment --keysend --d={node} --amt={amount}"""
            + """ --data 7629171="""
            + hex_encoded_message
        )

        input("\nContinue...")
    except:
        pass

def localchatnewB():
    lndconnectData= pickle.load(open("config/blndconnect.conf", "rb")) # Load the file 'bclock.conf'
    lndconnectload = lndconnectData # Copy the variable pathv to 'path'
    try:
        closed()
        print("\n\tRead.\n")
        os.system("""lncli listinvoices | grep "7629171" | tr -d '"' | tr -d ',' | sed 's/7629171/0a0a2d5079424c4f434b204d6573736167652052656365697665643a200a/g' | html2text | xxd -r -p | xargs --null""")
        input("\nContinue...")
    except:
        pass

def localchatlistB():
    lndconnectData= pickle.load(open("config/blndconnect.conf", "rb")) # Load the file 'bclock.conf'
    lndconnectload = lndconnectData # Copy the variable pathv to 'path'
    try:
        closed()
        print("\n\tList.\n")
        os.system("""lncli listpayments | grep "7629171" | tr -d '"' | tr -d ',' | sed 's/7629171/0a0a202d5079424c4f434b204d6573736167653a200a/g' | html2text | xxd -r -p | xargs --null""")
        input("\nContinue...")
    except:
        pass

def localchatsendC():
    lndconnectData= pickle.load(open("config/blndconnect.conf", "rb")) # Load the file 'bclock.conf'
    lndconnectload = lndconnectData # Copy the variable pathv to 'path'
    try:
        closed()
        print("\n\tWrite.\n")
        lncli = " sendpayment "
        node = input("Send to NodeID: ")
        amount = input("Amount in sats: ")
        message = input("Message: ")
        encoded_message = message.encode('utf-8')
        hex_encoded_message = encoded_message.hex()
        print(encoded_message.hex())
        input("\nContinue...")

        while True:
            if amount in ["", "0"]:
                amount = input("\nAmount in sats: ")
            else:
                break
        os.system(
            f"""lncli sendpayment --keysend --d={node} --amt={amount}"""
            + """ --data 34343434="""
            + hex_encoded_message
        )

        input("\nContinue...")
    except:
        pass

def localchatnewC():
    lndconnectData= pickle.load(open("config/blndconnect.conf", "rb")) # Load the file 'bclock.conf'
    lndconnectload = lndconnectData # Copy the variable pathv to 'path'
    try:
        closed()
        print("\n\tRead.\n")
        os.system("""lncli listinvoices | grep "34343434" | tr -d '"' | tr -d ',' | sed 's/34343434/0a0a2d5079424c4f434b204d6573736167652052656365697665643a200a/g' | html2text | xxd -r -p | xargs --null""")
        input("\nContinue...")
    except:
        pass

def localchatlistC():
    lndconnectData= pickle.load(open("config/blndconnect.conf", "rb")) # Load the file 'bclock.conf'
    lndconnectload = lndconnectData # Copy the variable pathv to 'path'
    try:
        closed()
        print("\n\tList.\n")
        lncli = " listpayments "
        os.system("""lncli listpayments | grep "34343434" | tr -d '"' | tr -d ',' | sed 's/34343434/0a0a202d5079424c4f434b204d6573736167653a200a/g' | html2text | xxd -r -p | xargs --null""")
        input("\nContinue...")
    except:
        pass

def localchannelbalance():
    lndconnectData= pickle.load(open("config/blndconnect.conf", "rb")) # Load the file 'bclock.conf'
    lndconnectload = lndconnectData # Copy the variable pathv to 'path'
    lncli = " channelbalance"
    lsd = os.popen(lndconnectload['ln'] + lncli).read()
    lsd0 = str(lsd)
    d = json.loads(lsd0)
    print("""
    ---------------------------------------------------------

    \tLOCAL CHANNEL BALANCE

    Balance: {} sats
    Pending Channels: {} sats

    ---------------------------------------------------------
    """.format(d['balance'], d['pending_open_balance']))
    input("\nContinue... ")

def localnewaddress():
    lndconnectData= pickle.load(open("config/blndconnect.conf", "rb")) # Load the file 'bclock.conf'
    lndconnectload = lndconnectData # Copy the variable pathv to 'path'
    lncli = " newaddress p2wkh"
    lsd = os.popen(lndconnectload['ln'] + lncli).read()
    lsd0 = str(lsd)
    d = json.loads(lsd0)
    qr = qrcode.QRCode(
    version=1,
    error_correction=qrcode.constants.ERROR_CORRECT_L,
    box_size=10,
    border=4,
    )
    print("\033[1;30;47m")
    qr.add_data(d['address'])
    qr.print_ascii()
    print("\033[0;37;40m")
    qr.clear()
    print("Bitcoin Address: " + d['address'])
    input("\nContinue... ")

def localbalanceOC():
    lndconnectData= pickle.load(open("config/blndconnect.conf", "rb")) # Load the file 'bclock.conf'
    lndconnectload = lndconnectData # Copy the variable pathv to 'path'
    lncli = " walletbalance"
    lsd = os.popen(lndconnectload['ln'] + lncli).read()
    lsd0 = str(lsd)
    d = json.loads(lsd0)
    print("\n----------------------------------------------------------------------------------------------------")
    print("\n\tLOCAL ONCHAIN BALANCE\n")
    print("Total Balance: " + d['total_balance'] + " sats")
    print("Confirmed Balance: " + d['confirmed_balance'] + " sats")
    print("Unconfirmed Balance: " + d['unconfirmed_balance'] + " sats")
    print("----------------------------------------------------------------------------------------------------\n")
    input("\nContinue... ")


def localrebalancelnd():
    lndconnectData= pickle.load(open("config/blndconnect.conf", "rb")) # Load the file 'bclock.conf'
    lndconnectload = lndconnectData # Copy the variable pathv to 'path'
    lncli = " listchannels"
    while True:
        lsd = os.popen(lndconnectload['ln'] + lncli).read()
        lsd0 = str(lsd)
        d = json.loads(lsd0)
        n = d['channels']
        clear()
        print("\033[1;32;40m")
        blogo()
        print("\033[0;37;40m")
        print("<<< Back to the Main Menu Press Control + C.\n\n")
        print("\t\nChannels\n")
        try:
            print("""\n\tLIST CHANNELS TO REBALANCE\n
                                 \t\033[1;32;40mLOCAL\033[0;37;40m BALANCE \t\033[1;31;40mREMOTE\033[0;37;40m BALANCE
            """)

            for item in n:
                s = item
                if int(s['local_balance']) >= int(s['remote_balance']):
                    total = int(s['local_balance']) - int(s['remote_balance'])
                elif int(s['local_balance']) <= int(s['remote_balance']):
                    total = int(s['remote_balance']) - int(s['local_balance'])
                print("Node ID: " + str(s['chan_id']) + "\t\033[1;32;40m " + str(s['local_balance']) + "\033[0;37;40m sats \033[1;31;40m\t" + str(s['remote_balance']) + "\033[0;37;40m sats \033[3;33;40m" + "\tDIFFERENCE: {}\033[0;37;40m sats".format(str(total)) )
            fromnode = input("\nSelect FROM a Node ID : ")
            tonode = input("\nSelect TO a Node ID : ")
            amt = input("\nAmount in sats: ")
            fee = input("\nMax Fee factor in sats: ")
            fromtonode = "python3 rebalance.py -f {} -t {} -a {} --max-fee-factor {}".format(fromnode,tonode,amt,fee)
            os.system(str(fromtonode))
            input("Continue...")
        except:
            break

# Remote connection with rest -------------------------------------

def getnewinvoice():
    lndconnectData= pickle.load(open("config/blndconnect.conf", "rb")) # Load the file 'bclock.conf'
    lndconnectload = lndconnectData # Copy the variable pathv to 'path'
    cert_path = lndconnectload["tls"]
    macaroon = codecs.encode(open(lndconnectload["macaroon"], 'rb').read(), 'hex')
    headers = {'Grpc-Metadata-macaroon': macaroon}
    qr = qrcode.QRCode(
    version=1,
    error_correction=qrcode.constants.ERROR_CORRECT_L,
    box_size=10,
    border=4,
    )
    try:
        amount = input("Amount in sats: ")
        memo = input("Memo: ")
        url = 'https://{}/v1/invoices'.format(lndconnectload["ip_port"])
        data = {

            }
        if amount == "":
            r = requests.post(
                url,
                headers=headers,
                verify=cert_path,
                json={"memo": f'{memo} -PyBLOCK'},
            )

        else:
            r = requests.post(
                url,
                headers=headers,
                verify=cert_path,
                json={"value": amount, "memo": f'{memo} -PyBLOCK'},
            )


        a = r.json()
        print("\033[1;30;47m")
        qr.add_data(a['payment_request'])
        qr.print_ascii()
        print("\033[0;37;40m")
        qr.clear()
        print("Lightning Invoice: " + a['payment_request'])
        b = str(a['payment_request'])
        while True:
            url = 'https://{}/v1/payreq/{}'.format(lndconnectload["ip_port"], b)
            r = requests.get(url, headers=headers, verify=cert_path)
            a = r.json()
            url = 'https://{}/v1/invoice/{}'.format(lndconnectload["ip_port"],a['payment_hash'])
            rr = requests.get(url, headers=headers, verify=cert_path)
            m = rr.json()
            if m['state'] == 'SETTLED':
                print("\033[1;32;40m")
                clear()
                blogo()
                tick()
                print("\033[0;37;40m")
                t.sleep(2)
                break
            elif m['state'] == 'CANCELED':
                print("\033[1;31;40m")
                clear()
                blogo()
                canceled()
                print("\033[0;37;40m")
                t.sleep(2)
                break
    except:
        pass

def payinvoice():
    lndconnectData= pickle.load(open("config/blndconnect.conf", "rb")) # Load the file 'bclock.conf'
    lndconnectload = lndconnectData # Copy the variable pathv to 'path'
    cert_path = lndconnectload["tls"]
    macaroon = codecs.encode(open(lndconnectload["macaroon"], 'rb').read(), 'hex')
    headers = {'Grpc-Metadata-macaroon': macaroon}
    try:
        while True:
            bolt11N = input("Insert the invoice to pay: ")
            url = 'https://{}/v1/payreq/{}'.format(lndconnectload["ip_port"],bolt11N)
            r = requests.get(url, headers=headers, verify=cert_path)
            s = r.json()
            print("\n----------------------------------------------------------------------------------------------------")
            print("""
            \tINVOICE DECODED
            Destination: {}
            Payment Hash: {}
            Amount: {} sats
            Description: {}
            """.format(s['destination'], s['payment_hash'], s['num_satoshis'], s['description']))
            print("----------------------------------------------------------------------------------------------------\n")
            print("<<< Cancel Control + C")
            input("\nEnter to Continue... ")
            bolt11 = bolt11N.lower()
            r = requests.post(
                url='https://{}/v1/channels/transactions'.format(lndconnectload["ip_port"]), headers=headers, verify=cert_path, json={"payment_request": bolt11}
            )
            try:
                r.json()['error']
                print("\nThe Invoice don't have an amount. Please insert an Invoice with amount. \n")
                continue
            except:
                break
        ok, checking_id, fee_msat, error_message = r.ok, None, 0, None
        r = requests.get(url='https://{}/v1/payreq/{}'.format(lndconnectload["ip_port"],bolt11), headers=headers, verify=cert_path,)
        t.sleep(5)
        if r.ok:
            checking_id = r.json()["payment_hash"]
            print("\033[1;32;40m")
            clear()
            blogo()
            tick()
        else:
            error_message = r.json()["error"]
            print("\033[1;31;40m")
            clear()
            blogo()
            canceled()
        print("\033[0;37;40m")
        t.sleep(2)
    except:
        pass

def getnewaddress():
    lndconnectData= pickle.load(open("config/blndconnect.conf", "rb")) # Load the file 'bclock.conf'
    lndconnectload = lndconnectData # Copy the variable pathv to 'path'
    cert_path = lndconnectload["tls"]
    macaroon = codecs.encode(open(lndconnectload["macaroon"], 'rb').read(), 'hex')
    headers = {'Grpc-Metadata-macaroon': macaroon}
    qr = qrcode.QRCode(
    version=1,
    error_correction=qrcode.constants.ERROR_CORRECT_L,
    box_size=10,
    border=4,
    )
    try:
        url = 'https://{}/v1/newaddress'.format(lndconnectload["ip_port"])
        r = requests.get(url, headers=headers, verify=cert_path)
        addr = r.json()
        print("\033[1;30;47m")
        qr.add_data(addr['address'])
        qr.print_ascii()
        print("\033[0;37;40m")
        print("Bitcoin Address: " + addr['address'])
        qr.clear()
        input("\nContinue... ")
    except:
        pass

def listinvoice():
    lndconnectData= pickle.load(open("config/blndconnect.conf", "rb")) # Load the file 'bclock.conf'
    lndconnectload = lndconnectData # Copy the variable pathv to 'path'
    qr = qrcode.QRCode(
    version=1,
    error_correction=qrcode.constants.ERROR_CORRECT_L,
    box_size=10,
    border=4,
    )
    cert_path = lndconnectload["tls"]
    macaroon = codecs.encode(open(lndconnectload["macaroon"], 'rb').read(), 'hex')
    headers = {'Grpc-Metadata-macaroon': macaroon}
    url = 'https://{}/v1/invoices'.format(lndconnectload["ip_port"])
    r = requests.get(url, headers=headers, verify=cert_path)
    a = r.json()
    n = a['invoices']
    while True:
        clear()
        print("\033[1;32;40m")
        blogo()
        print("\033[0;37;40m")
        print("<<< Back to the Main Menu Press Control + C.\n\n")
        print("\tInvoices\n")
        try:
            print("\n\tLIST INVOICES\n")
            for r in range(len(n)):
                s = n[r]
                print("Invoice: " + s['r_hash'] + " " + s['state'])

            nd = input("\nSelect RHash: ")
            for item in n:
                s = item
                nn = s['r_hash']
                if nd == nn:
                    print("\n----------------------------------------------------------------------------------------------------")
                    print("""
                    \tINVOICE DECODED
                    Memo: {}
                    Invoice: {}
                    Amount: {} sats
                    State: {}
                    """.format(s['memo'], s['payment_request'], s['amt_paid_sat'], s['state']))
                    print("----------------------------------------------------------------------------------------------------\n")
                    print("\033[1;30;47m")
                    qr.add_data(s['payment_request'])
                    qr.print_ascii()
                    print("\033[0;37;40m")
                    qr.clear()
            input("\nContinue... ")
        except:
            break
    input("\nContinue... ")

def getinfo():
    lndconnectData= pickle.load(open("config/blndconnect.conf", "rb")) # Load the file 'bclock.conf'
    lndconnectload = lndconnectData # Copy the variable pathv to 'path'
    qr = qrcode.QRCode(
    version=1,
    error_correction=qrcode.constants.ERROR_CORRECT_L,
    box_size=10,
    border=4,
    )
    cert_path = lndconnectload["tls"]
    macaroon = codecs.encode(open(lndconnectload["macaroon"], 'rb').read(), 'hex')
    headers = {'Grpc-Metadata-macaroon': macaroon}
    url = 'https://{}/v1/getinfo'.format(lndconnectload["ip_port"])
    r = requests.get(url, headers=headers, verify=cert_path)
    a = r.json()
    hash = a['identity_pubkey']
    rh = Robohash(hash)
    rh.assemble(roboset='set1')
    if not os.path.isfile(str(f'{hash}.png')):
        with open(f'{hash}.png', "wb") as f:
            rh.img.save(f, format="png")

        img_path = open(f'{hash}.png', "rb")
        img = Image.open(img_path)

        h = 20
        w = int((img.width / img.height) * 50)

        img = img.resize((w,h), Image.ANTIALIAS)
        img_arr = np.asarray(img)
        h,w,c = img_arr.shape

    img_path = open(f'{hash}.png', "rb")
    img = Image.open(img_path)

    h = 20
    w = int((img.width / img.height) * 50)

    img = img.resize((w,h), Image.ANTIALIAS)
    img_arr = np.asarray(img)
    h,w,c = img_arr.shape

    for x in range(h):
        for y in range(w):
            pix = img_arr[x][y]
            print(get_color(pix[0], pix[1], pix[2]), sep='', end='')
        print()
    print("\n----------------------------------------------------------------------------------------------------")
    print("""
    \t NODE INFORMATION
    Version: {}
    Node ID: {}
    Alias: {}
    Color: {}
    Pending Channels: {}
    Active Channels: {}
    Inactive Channels: {}
    Peers: {}
    URLS: {}
    """.format(a['version'], a['identity_pubkey'], a['alias'], a['color'], a['num_pending_channels'], a['num_active_channels'], a['num_inactive_channels'], a['num_peers'], a['uris']))
    print("\033[1;30;47m")
    qr.add_data(a['identity_pubkey'])
    qr.print_ascii()
    print("\033[0;37;40m")
    qr.clear()
    print("----------------------------------------------------------------------------------------------------\n")
    input("\nContinue... ")

#--------------------------------- NYMs -----------------------------------

def get_ansi_color_code(r, g, b):
    if r == g == b:
        if r < 8:
            return 16
        if r > 248:
            return 231
        return round(((r - 8) / 247) * 24) + 232
    return 16 + (36 * round(r / 255 * 5)) + (6 * round(g / 255 * 5)) + round(b / 255 * 5)


def get_color(r, g, b):
    return "\x1b[48;5;{}m \x1b[0m".format(int(get_ansi_color_code(r,g,b)))

def channels():
    lndconnectData= pickle.load(open("config/blndconnect.conf", "rb")) # Load the file 'bclock.conf'
    lndconnectload = lndconnectData # Copy the variable pathv to 'path'
    cert_path = lndconnectload["tls"]
    macaroon = codecs.encode(open(lndconnectload["macaroon"], 'rb').read(), 'hex')
    headers = {'Grpc-Metadata-macaroon': macaroon}
    url = 'https://{}/v1/channels'.format(lndconnectload["ip_port"])
    r = requests.get(url, headers=headers, verify=cert_path)
    a = r.json()
    n = a['channels']
    while True:
        clear()
        print("\033[1;32;40m")
        blogo()
        print("\033[0;37;40m")
        print("<<< Back to the Main Menu Press Control + C.\n\n")
        print("\t\nChannels\n")
        try:
            print("\n\tLIST CHANNELS\n")
            for r in range(len(n)):
                s = n[r]
                hash = s['remote_pubkey']
                rh = Robohash(hash)
                rh.assemble(roboset='set1')
                if not os.path.isfile(str(f'{hash}.png')):
                    with open(f'{hash}.png', "wb") as f:
                        rh.img.save(f, format="png")

                    img_path = open(f'{hash}.png', "rb")
                    img = Image.open(img_path)

                    h = 1
                    w = int((img.width / img.height) * 5)

                    img = img.resize((w,h), Image.ANTIALIAS)
                    img_arr = np.asarray(img)
                    h,w,c = img_arr.shape

                img_path = open(f'{hash}.png', "rb")
                img = Image.open(img_path)

                h = 1
                w = int((img.width / img.height) * 5)

                img = img.resize((w,h), Image.ANTIALIAS)
                img_arr = np.asarray(img)
                h,w,c = img_arr.shape

                for x in range(h):
                    for y in range(w):
                        pix = img_arr[x][y]
                        print(get_color(pix[0], pix[1], pix[2]), sep='', end='')
                    print()
                print("Node ID: " + s['remote_pubkey'])

            nd = input("\nSelect a Node ID: ")
            for item in n:
                s = item
                nn = s['remote_pubkey']
                if nd == nn:
                    hash = s['remote_pubkey']
                    rh = Robohash(hash)
                    rh.assemble(roboset='set1')

                    img_path = open(f'{hash}.png', "rb")
                    img = Image.open(img_path)

                    h = 20
                    w = int((img.width / img.height) * 50)

                    img = img.resize((w,h), Image.ANTIALIAS)
                    img_arr = np.asarray(img)
                    h,w,c = img_arr.shape

                    for x in range(h):
                        for y in range(w):
                            pix = img_arr[x][y]
                            print(get_color(pix[0], pix[1], pix[2]), sep='', end='')
                        print()
                    print("\n----------------------------------------------------------------------------------------------------")
                    print("""
                    \tCHANNEL DECODED
                    Active: {}
                    Node ID: {}
                    Channel Point: {}
                    Channel Capacity: {} sats
                    Local Balance: {} sats
                    Remote Balance: {} sats
                    Total Sent: {} sats
                    Total Received: {} sats
                    """.format(s['active'], s['remote_pubkey'], s['channel_point'], s['capacity'], s['local_balance'], s['remote_balance'], s['total_satoshis_sent'], s['total_satoshis_received']))
                    print("----------------------------------------------------------------------------------------------------\n")

            input("\nContinue... ")
        except:
            break

def channelbalance():
    lndconnectData= pickle.load(open("config/blndconnect.conf", "rb")) # Load the file 'bclock.conf'
    lndconnectload = lndconnectData # Copy the variable pathv to 'path'
    cert_path = lndconnectload["tls"]
    macaroon = codecs.encode(open(lndconnectload["macaroon"], 'rb').read(), 'hex')
    headers = {'Grpc-Metadata-macaroon': macaroon}
    url = 'https://{}/v1/balance/channels'.format(lndconnectload["ip_port"])
    r = requests.get(url, headers=headers, verify=cert_path)
    a = r.json()
    print("""
    ---------------------------------------------------------

    \tLOCAL CHANNEL BALANCE

    Balance: {} sats
    Pending Channels: {} sats

    ---------------------------------------------------------
    """.format(a['balance'], a['pending_open_balance']))
    input("\nContinue... ")

def listonchaintxs():
    qr = qrcode.QRCode(
    version=1,
    error_correction=qrcode.constants.ERROR_CORRECT_L,
    box_size=10,
    border=4,
    )
    cert_path = lndconnectload["tls"]
    macaroon = codecs.encode(open(lndconnectload["macaroon"], 'rb').read(), 'hex')
    headers = {'Grpc-Metadata-macaroon': macaroon}
    url = 'https://{}/v1/transactions'.format(lndconnectload["ip_port"])
    r = requests.get(url, headers=headers, verify=cert_path)
    a = r.json()
    n = a['transactions']
    while True:
        clear()
        print("\033[1;32;40m")
        blogo()
        print("\033[0;37;40m")
        print("<<< Back to the Main Menu Press Control + C.\n\n")
        print("\t\nTransactions\n")
        try:
            print("\n\tLIST ONCHAIN TRANSACTIONS\n")
            for r in range(len(n)):
                s = n[r]
                print("Transaction Hash: " + " " + s['tx_hash'] + " sats")
            nd = input("\nSelect RHash: ")

            for item in n:
                s = item
                nn = s['tx_hash']
                trx = s['dest_addresses']
                if nd == nn:
                    print("\n----------------------------------------------------------------------------------------------------")
                    print("""
                    \tONCHAIN TRANSACTION DECODED
                    Amount: {} sats
                    Tx Hash: {}
                    Block Hash: {}
                    Block Height: {}
                    Confirmations: {}
                    Destination: {}
                    """.format(s['amount'], s['tx_hash'], s['block_hash'], s['block_height'], s['num_confirmations'], trx))
                    print("----------------------------------------------------------------------------------------------------\n")
                    print("\nTransaction Hash")
                    print("\033[1;30;47m")
                    qr.add_data(s['tx_hash'])
                    qr.print_ascii()
                    print("\033[0;37;40m")
                    qr.clear()
            input("\nContinue... ")
        except:
            break

def balanceOC():
    lndconnectData= pickle.load(open("config/blndconnect.conf", "rb")) # Load the file 'bclock.conf'
    lndconnectload = lndconnectData # Copy the variable pathv to 'path'
    cert_path = lndconnectload["tls"]
    macaroon = codecs.encode(open(lndconnectload["macaroon"], 'rb').read(), 'hex')
    headers = {'Grpc-Metadata-macaroon': macaroon}
    url = 'https://{}/v1/balance/blockchain'.format(lndconnectload["ip_port"])
    r = requests.get(url, headers=headers, verify=cert_path)
    a = r.json()
    print("\n----------------------------------------------------------------------------------------------------")
    print("\n\tLOCAL ONCHAIN BALANCE\n")
    print("Total Balance: " + a['total_balance'] + " sats")
    print("Confirmed Balance: " + a['confirmed_balance'] + " sats")
    print("Unconfirmed Balance: " + a['unconfirmed_balance'] + " sats")
    print("----------------------------------------------------------------------------------------------------\n")
    input("\nContinue... ")

# END Remote connection with rest -------------------------------------
#---------------------------------OPENDIME-----------------------------

def ADDRbalance():
    import os, sys; sys.path.insert(0, os.path.normpath(__file__ + '/support/pycode.zip'))
    import support.pycode.od_wallet; support.pycode.od_wallet.main()



#Developer: Curly60e
#PyBLOCK its a clock of the Bitcoin blockchain.


import os
import os.path
import time as t


def gitclone():
    url = "https://github.com/curly60e/satellite"
    os.system("git clone " + url)
    os.system("mkdir satellite/api/examples/.gnupg")
    os.system("gpg --full-generate-key --homedir satellite/api/examples/.gnupg")

def satnode():
    try:
        os.system("python3 satellite/api/examples/demo-rx.py &")
        t.sleep(5)
        os.system("python3 satellite/api/examples/api_data_reader.py --demo  --plaintext ")
    except:
        os.system("ps -ef | grep api_data_reader.py | grep -v grep | awk '{print $2}' | xargs kill -9")
        os.system("ps -ef | grep demo-rx.py | grep -v grep | awk '{print $2}' | xargs kill -9")

def matrixsc():
    if os.path.isdir('$HOME/pyblock/terminal_matrix'):
        print("OK Pass")
    else:
        url = "https://github.com/curly60e/terminal_matrix.git"
        os.system("git clone " + url)



#Developer: Curly60e
#PyBLOCK its a clock of the Bitcoin blockchain.

import os
import psutil
import time as t
from pblogo import *


def clear(): # clear the screen
    os.system('cls' if os.name=='nt' else 'clear')

def sysinfoDetail(): #Cpu and memory usage
# gives a single float value
    while True:
        try:
            clear()
            blogo()
            print("   \033[0;37;40m----------------------------")
            print("   \033[3;33;40mCPU Usage: \033[1;32;40m" + str(psutil.cpu_percent()) + "\033[0;37;40m%")
            print("   \033[3;33;40mMemory Usage: \033[1;32;40m" "{}\033[0;37;40m%".format(int(psutil.virtual_memory().percent)))
            print("   \033[3;33;40mMemory Available: \033[1;32;40m" "{} \033[0;37;40mMB".format(int(psutil.virtual_memory().available / 1024 / 1024)))
            print("   \033[3;33;40mDisk Usage: \033[1;32;40m" "{}%\033[0;37;40m%".format(psutil.disk_usage('/').percent))
            print("   \033[0;37;40m----------------------------")
            t.sleep(1)
        except:
            break



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pybitblock',
 'pybitblock.SPV',
 'pybitblock.SPV.grpc_generated',
 'pybitblock.SPV.terminal_matrix',
 'pybitblock.grpc_generated',
 'pybitblock.terminal_matrix']

package_data = \
{'': ['*'],
 'pybitblock': ['images/*', 'resources/images/*'],
 'pybitblock.SPV': ['config/*', 'images/*', 'resources/images/*']}

install_requires = \
['Pillow>=8.4,<10.0',
 'art>=5.3,<6.0',
 'certifi>=2021.10.8,<2022.0.0',
 'chardet>=4.0.0,<5.0.0',
 'googleapis-common-protos==1.52.0',
 'html2text==2020.1.16',
 'idna>=3.3,<4.0',
 'jq==1.2.2',
 'lnpay-py>=0.1.0,<0.2.0',
 'numpy>=1.21.4,<2.0.0',
 'pdf2text==1.0.0',
 'pdf2txt==0.7.3',
 'pdfminer>=20191125,<20191126',
 'protobuf==3.15.0',
 'psutil>=5.8.0,<6.0.0',
 'pycoingecko>=2.2.0,<3.0.0',
 'python-cfonts>=1.5.2,<2.0.0',
 'python-gnupg>=0.4.7,<0.5.0',
 'qrcode>=7.3.1,<8.0.0',
 'requests>=2.26.0,<3.0.0',
 'robohash>=1.1,<2.0',
 'simplejson>=3.17.6,<4.0.0',
 'six==1.15.0',
 'sseclient-py>=1.7.2,<2.0.0',
 'termcolor>=1.1.0,<2.0.0',
 'typer>=0.4.0,<0.5.0',
 'urllib3>=1.26.7,<2.0.0',
 'xmltodict>=0.12.0,<0.13.0']

entry_points = \
{'console_scripts': ['pyblock = pybitblock.console:main']}

setup_kwargs = {
    'name': 'pybitblock',
    'version': '2.0.1',
    'description': 'Python Bitcoin block dashboard, transactions, send message to Space and more',
    'long_description': '<meta name="lightning" content="pyblock@zbd.gg" />\n<meta property="og:image" content="./pybitblock/resources/images/Logo.PNG" />\n\n<img src="./pybitblock/resources/images/Logo.PNG" width="80%" />\n\n   \n    ----------------------\n    CPU Usage: X% \n    Memory Usage: X% \n    ----------------------\n\t\n    Local: PyBLOCK\n    Node:  XxXxXxX\n    Block: XxXxXxX\n    Version: X.x.X\n    \n    A. PyBLOCK\n    B. Bitcoin Core\n    L. Lightning Network\n    P. Platforms\n    S. Settings\n    X. Donate\n    Q. Exit\n    \n    Select option:     \n\n\n<br />\n\n<img src="https://pbs.twimg.com/media/Entgg9HXEAI6yea.jpg" width="50%" />\n\n<br />\n\n<img src="https://pbs.twimg.com/media/EpYxp_oWEAE88s-.jpg" width="50%" />\n\n<br />\n\n<img src="https://pbs.twimg.com/media/E5VnuZYXIAAVlVh.jpg" width="50%" />\n\n<br />\n\n<img src="https://pbs.twimg.com/media/FDBefZPWQAQDEVo.jpg" width="50%" />\n\n<br />\n\n<img src="https://pbs.twimg.com/media/FAskRpLXIAgC-fC.jpg" width="50%" />\n\n<br />\n\n<img src="https://pbs.twimg.com/media/FNNUNIWXwAAajhL.jpg" width="50%" />\n\n<br />\n\n<img src="https://pbs.twimg.com/media/FADnL4DXoAIlkwR.jpg" width="50%" />\n\n<br />\n\n# PyBLOCK\nℙ𝕪𝔹𝕃𝕆ℂ𝕂 𝕚𝕥𝕤 𝕒 𝔹𝕚𝕥𝕔𝕠𝕚𝕟 𝔻𝕒𝕤𝕙𝕓𝕠𝕒𝕣𝕕 𝕨𝕚𝕥𝕙 ℂ𝕪𝕡𝕙𝕖𝕣𝕡𝕦𝕟𝕜 𝕒𝕖𝕤𝕥𝕙𝕖𝕥𝕚𝕔.\n\n- This will fully work on a Node that has Bitcoin Core and LND installed.\n- We fully tested and worked perfect on [MyNodeBTC](https://twitter.com/_PyBlock_/status/1402516068959199233)\n- We fully tested and worked perfect on [RaspiBlitz](https://twitter.com/_PyBlock_/status/1405788110458441728)\n- We fully tested and worked perfect on [BitcoinMachines](https://twitter.com/_PyBlock_/status/1365757861217861632)\n- We fully tested and worked perfect on [Umbrel](https://twitter.com/_PyBlock_/status/1405574038320201733)\n\n# First Start\n\n- You will need to find the path of the files tls.cert and admin.macaroon to do the REST connection to have access to LND.\n- [Poetry](https://python-poetry.org/) is needed to ensure every user has the same python dependencies installed.\n\n    ### From LOCAL Node\n\n    Open the Terminal.\n\n    -- Easy mode:\n    * a@A:~> sudo apt install hexyl\n    * a@A:~> pip3 install pybitblock\n    * a@A:~> pyblock\n    \n    -- Manual mode:\n    * a@A:~> sudo apt install hexyl\n    * a@A:~> pip3 install poetry\n    * a@A:~> git clone https://github.com/curly60e/pyblock.git\n    * a@A:~> cd pyblock\n    * a@A:~> poetry install\n    * a@A:~> cd pybitblock\n    * a@A:~> poetry run python3 PyBlock.py\n\n    -- Upgrade:\n    * a@A:~> pip3 install pybitblock -U\n    * a@A:~> pyblock\n\n    <br />\n\n    - This is how we continue.\n\n    <br />\n\n    <img src="./pybitblock/resources/images/REST1.PNG" width="30%" />\n\n    <br />\n\n    - It will ask you for the IP:PORT (REST PORT) in this case use: localhost instead of the IP.\n\n    <br />\n\n    <img src="./pybitblock/resources/images/REST2.PNG" width="30%" />\n\n    <br />\n\n    - Then it will ask you for the path to the tls.cert.\n\n    <br />\n\n    <img src="./pybitblock/resources/images/REST3.PNG" width="30%" />\n\n    <br />\n\n    - Then it will ask you for the path to the admin.macaroon.\n\n    <br />\n\n    <img src="./pybitblock/resources/images/REST4.PNG" width="30%" />\n\n    <br />  \n\n    - Then it will ask you for the path to bitcoin-cli or if you have already installed just put: bitcoin-cli.\n\n    <br />\n\n    <img src="./pybitblock/resources/images/bitcoin-cli.PNG" width="30%" />\n\n    <br />\n\n    - And you are in.\n\n    <br />\n\n    <img src="./pybitblock/resources/images/main.PNG" width="30%" />\n\n    <br />\n\n    ### From REMOTE Computer\n\n    * You will need to have tls.cert and admin.macaroon already downloaded from your LND node.\n    * [Poetry](https://python-poetry.org/) is needed to ensure every user has the same python dependencies installed.\n\n     - Open the Terminal\n     \n\t    -- Easy mode:\n\t    * a@A:~> sudo apt install hexyl\n\t    * a@A:~> pip3 install pybitblock\n\t    * a@A:~> pyblock\n\n\t    -- Manual mode:\n\t    * a@A:~> sudo apt install hexyl\n\t    * a@A:~> pip3 install poetry\n\t    * a@A:~> git clone https://github.com/curly60e/pyblock.git\n\t    * a@A:~> cd pyblock\n\t    * a@A:~> poetry install\n\t    * a@A:~> cd pybitblock\n\t    * a@A:~> poetry run python3 PyBlock.py\n\n\t    -- Upgrade:\n\t    * a@A:~> pip3 install pybitblock -U\n\t    * a@A:~> pyblock\n\n        <br />\n\n        - This is how we continue.\n\n        <br />\n\n        <img src="./pybitblock/resources/images/REST1.PNG" width="30%" />\n\n        <br />\n\n        - It will ask you for the IP:PORT (REST PORT).\n\n        <br />\n\n        <img src="./pybitblock/resources/images/REST1REMOTE.PNG" width="30%" />\n\n        <br />\n\n        - Then it will ask you for the path to the tls.cert.\n\n        <br />\n\n        <img src="./pybitblock/resources/images/REST2REMOTE.PNG" width="30%" />\n\n        <br />\n\n        - Then it will ask you for the path to the admin.macaroon.\n\n        <br />\n\n        <img src="./pybitblock/resources/images/REST3REMOTE.PNG" width="30%" />\n\n        <br />  \n\n        - Then it will ask you for the path to bitcoin-cli or if you have already installed just put: bitcoin-cli.\n\n        <br />\n\n        <img src="./pybitblock/resources/images/bitcoin-cli.PNG" width="30%" />\n\n        <br />\n\n        - And you are in.\n\n        <br />\n\n        <img src="./pybitblock/resources/images/main.PNG" width="30%" />\n\n        <br />\n\n\n## Dependencies\n\n  - Install Curl on Debian based type:\n    - sudo apt install curl\n\n## How to execute\n\n  - python3 PyBlock.py\n\n\n### Created by\n\n[@Curly60e](https://twitter.com/curly60e) ⚡️ curly60e@bitrefill.com\n\n### Pentester and contribution by\n\n[@SN](https://twitter.com/__B__T__C__) ⚡️ sn@getalby.com\n\n### Contributor\n\n[@Danvergara](https://twitter.com/__danvergara__)\n\n### Tools by\n\n[@SamouraiDev,](https://twitter.com/SamouraiDev)\n[@Korynewton,](https://twitter.com/kn3wt)\n[@Tippin_Me,](https://twitter.com/tippin_me)\n[@TallyCoinApp,](https://twitter.com/tallycoinapp)\n[@DJBooth007,](https://twitter.com/djbooth007)\n[@MemPool,](https://twitter.com/mempool)\n[@CoinGecko,](https://twitter.com/coingecko)\n[@Igor_Chubin,](https://twitter.com/igor_chubin)\n[@Shesek,](https://twitter.com/shesek)\n[@LNBits,](https://twitter.com/lnbits)\n[@LNPAYco,](https://twitter.com/LNPAYco)\n[@OpenNodeCo,](https://twitter.com/OpenNodeCo)\n[@BlockStream,](https://twitter.com/Blockstream)\n[@Gwidion,](https://twitter.com/gwidion)\n[@AlphaaZeta,](https://twitter.com/alphaazeta)\n[@Hampus_S,](https://twitter.com/hampus_s)\n[@Mutatrum,](https://twitter.com/mutatrum)\n[@RoboHash,](https://twitter.com/Robohash)\n[@C_Otto83,](https://twitter.com/c_otto83)\n[@BashCo_,](https://twitter.com/BashCo_)\n[@JamesOb,](https://twitter.com/jamesob)\n[@BenTheCarman,](https://twitter.com/benthecarman)\n[@Whale_Alert,](https://twitter.com/whale_alert)\n[@BitcoinExplorer,](https://twitter.com/BitcoinExplorer)\n[@JanoSide,](https://twitter.com/janoside)\n[@LNstats,](https://twitter.com/LNstats)\n[@Slush_Pool,](https://twitter.com/slush_pool)\n[@Braiins_Systems,](https://twitter.com/braiins_systems)\n[@JohnCantrell97,](https://twitter.com/JohnCantrell97)\n[@JoostJgr,](https://twitter.com/joostjgr)\n[@PRguitarman,](https://twitter.com/PRguitarman)\n[@NyanCat,](https://twitter.com/nyannyancat)\n...\n\nSUPPORT PyBLØCK.\n\n⚡️ pyblock@zbd.gg ⚡️\n\nBitcoin Address: bc1qjzaz34nv2ev55vfdu9m5qh0zq0fwcn6c7pkcrv\n\n<img src="/pybitblock/images/bitcoin-donation.png" width="30%" />\n\nSamourai Wallet Paynym: PM8TJhNTTq3YVocXuPtLjKx7pKkdUxqwTerWJ2j2a7dNitgyMmBPN6gK61yE17N2vgvQvKYokXktt6D6GZFTmocvDJhaUJfHt7ehEMmthjsT3NQHseFM\n\n<img src="/pybitblock/images/codeimage.png" width="30%" />\n\nMonero: 42jtb4dAfm6BQ8h6x56qGyAMMHVXGRwRMTSb2LwsBg1jVqD4TxfpD1pTK56tkrTMGhEKipZdDHfJrB1g8iQfvSyC7gZ9M8M\n\n<img src="/pybitblock/images/qrcode.png" width="20%" />\n',
    'author': 'curly60e',
    'author_email': 'curly60e@piserver.xyz',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/curly60e/pyblock',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<3.11',
}


setup(**setup_kwargs)



#! /usr/bin/env python3

# Author: Joao S. O. Bueno
# gwidion@gmail.com
# GPL. v3.0

MAX_CASCADES = 600
MAX_COLS = 20
FRAME_DELAY = 0.03

MAX_SPEED  = 5

import shutil, sys, time
from random import choice, randrange, paretovariate

CSI = "\x1b["
pr = lambda command: print("\x1b[", command, sep="", end="")
getchars = lambda start, end: [chr(i) for i in range(start, end)]

black, green, white = "30", "32", "37"

latin = getchars(0x30, 0x80)
greek = getchars(0x390, 0x3d0)
hebrew = getchars(0x5d0, 0x5eb)
cyrillic = getchars(0x400, 0x50)

chars= latin + greek + hebrew + cyrillic

def pareto(limit):
    scale = lines // 2
    number = (paretovariate(1.16) - 1) * scale
    return max(0, limit - number)

def init():
    global cols, lines
    cols, lines = shutil.get_terminal_size()
    pr("?25l")  # Hides cursor
    pr("s")  # Saves cursor position

def end():
    pr("m")   # reset attributes
    pr("2J")  # clear screen
    pr("u")  # Restores cursor position
    pr("?25h")  # Show cursor

def print_at(char, x, y, color="", bright="0"):
    pr("%d;%df" % (y, x))
    pr(bright + ";" + color + "m")
    print(char, end="", flush=True)

def update_line(speed, counter, line):
    counter += 1
    if counter >= speed:
        line += 1
        counter = 0
    return counter, line

def cascade(col):
    speed = randrange(1, MAX_SPEED)
    espeed = randrange(1, MAX_SPEED)
    line = counter = ecounter = 0
    oldline = eline = -1
    erasing = False
    bright = "1"
    limit = pareto(lines)
    while True:
        counter, line = update_line(speed , counter, line)
        if randrange(10 * speed) < 1:
            bright = "0"
        if line > 1 and line <= limit and oldline != line:
            print_at(choice(chars),col, line-1, green, bright)
        if line < limit:
            print_at(choice(chars),col, line, white, "1")
        if erasing:
            ecounter, eline = update_line(espeed, ecounter, eline)
            print_at(" ",col, eline, black)
        else:
            erasing = randrange(line + 1) > (lines / 2)
            eline = 0
        yield None
        oldline = line
        if eline >= limit:
            print_at(" ", col, oldline, black)
            break

def main():
    cascading = set()
    added_new = True
    while True:
        while add_new(cascading): pass
        stopped = iterate(cascading)
        sys.stdout.flush()
        cascading.difference_update(stopped)
        time.sleep(FRAME_DELAY)

def add_new(cascading):
    if randrange(MAX_CASCADES + 1) > len(cascading):
        col = randrange(cols)
        for i in range(randrange(MAX_COLS)):
            cascading.add(cascade((col + i) % cols))
        return True
    return False

def iterate(cascading):
    stopped = set()
    for c in cascading:
        try:
            next(c)
        except StopIteration:
            stopped.add(c)
    return stopped

def doit():
    try:
        init()
        main()
    except KeyboardInterrupt:
        pass
    finally:
        end()

if __name__=="__main__":
    doit()



#!/usr/bin/env python
# -*- coding: utf8 -*-


def btcSupplyAtBlock(b):
    if b >= 33 * 210000:
        return 20999999.9769
    reward = 50e8
    supply = 0
    y = 210000  # reward changes all y blocks
    while b > y - 1:
        supply = supply + y * reward
        reward = int(reward / 2.0)
        b = b - y
    supply = supply + b * reward
    return (supply + reward) / 1e8


if __name__ == "__main__":
    block = 777777  # you want the supply after which block?
    print(btcSupplyAtBlock(block))



import base64
import sys

MAX_ROUTES_TO_REQUEST = 100


def debug(message):
    sys.stderr.write(message + "\n")


def debugnobreak(message):
    sys.stderr.write(message)


class Routes:
    def __init__(self, lnd, payment_request, first_hop_channel, last_hop_channel):
        self.lnd = lnd
        self.payment_request = payment_request
        self.first_hop_channel = first_hop_channel
        self.last_hop_channel = last_hop_channel
        self.all_routes = []
        self.returned_routes = []
        self.ignored_edges = []
        self.ignored_nodes = []
        self.num_requested_routes = 0

    def has_next(self):
        self.update_routes()
        return self.returned_routes < self.all_routes

    def get_next(self):
        self.update_routes()
        for route in self.all_routes:
            if route not in self.returned_routes:
                self.returned_routes.append(route)
                return route
        return None

    def update_routes(self):
        while True:
            if self.returned_routes < self.all_routes:
                return
            if self.num_requested_routes >= MAX_ROUTES_TO_REQUEST:
                return
            self.request_route()

    def request_route(self):
        amount = self.get_amount()
        if self.last_hop_channel:
            last_hop_pubkey = self.last_hop_channel.remote_pubkey
        else:
            last_hop_pubkey = None
        if self.first_hop_channel:
            first_hop_channel_id = self.first_hop_channel.chan_id
        else:
            first_hop_channel_id = None
        routes = self.lnd.get_route(last_hop_pubkey, amount, self.ignored_edges,
                                    self.ignored_nodes, first_hop_channel_id)
        if routes is None:
            self.num_requested_routes = MAX_ROUTES_TO_REQUEST
        else:
            self.num_requested_routes += 1
            for route in routes:
                self.add_route(route)

    def add_route(self, route):
        if route is None:
            return
        if route not in self.all_routes:
            self.all_routes.append(route)

    @staticmethod
    def print_route(route):
        route_str = " -> ".join(str(h.chan_id) for h in route.hops)
        return route_str

    def get_amount(self):
        return self.payment_request.num_satoshis

    def ignore_first_hop(self, channel, show_message=True):
        own_key = self.lnd.get_own_pubkey()
        other_key = channel.remote_pubkey
        self.ignore_edge_from_to(channel.chan_id, own_key, other_key, show_message)

    def ignore_edge_on_route(self, failure_source_pubkey, route):
        ignore_next = False
        for hop in route.hops:
            if ignore_next:
                self.ignore_edge_from_to(hop.chan_id, failure_source_pubkey, hop.pub_key)
                return
            if hop.pub_key == failure_source_pubkey:
                ignore_next = True

    def ignore_node_with_highest_fee(self, route):
        max_fee_msat = 0
        max_fee_hop = None
        for hop in route.hops:
            if hop.fee_msat > max_fee_msat:
                max_fee_msat = hop.fee_msat
                max_fee_hop = hop

        pub_key = max_fee_hop.pub_key
        debugnobreak("High fees (%s msat), " % max_fee_msat)
        self.ignore_node(pub_key)

    def ignore_edge_from_to(self, chan_id, from_pubkey, to_pubkey, show_message=True):
        if show_message:
            debug("ignoring channel %s (from %s to %s)" % (chan_id, from_pubkey, to_pubkey))
        direction_reverse = from_pubkey > to_pubkey
        edge = {"channel_id": chan_id, "direction_reverse": direction_reverse}
        self.ignored_edges.append(edge)

    def ignore_node(self, pub_key):
        debug("ignoring node %s" % pub_key)
        self.ignored_nodes.append(base64.b16decode(pub_key, True))



#Developer: Curly60e
#PyBLOCK its a clock of the Bitcoin blockchain.

import os
import qrcode
import requests
import time as t
import sys
from nodeconnection import *
from pblogo import *
from logos import *

def clear(): # clear the screen
    os.system('cls' if os.name=='nt' else 'clear')

def apisender():
    qr = qrcode.QRCode(
    version=1,
    error_correction=qrcode.constants.ERROR_CORRECT_L,
    box_size=10,
    border=4,
    )
    message = input("\nInsert your Message: ")
    a = sys.getsizeof(message)
    mini = 6000
    n = (a * 50)
    if n <= mini:
        q = str(mini)
    elif n >= mini:
        k = n + mini
        q = str(k)

    url = 'https://api.blockstream.space/order'
    sentby = " - PyBLOCK."
    print("\n\tATENTION: YOU NEED TO PAY \033[1;31;40m" + q + "\033[0;37;40m MilliSats")
    amountmsat = input("\nInsert the amount in MSats: ")
    curl = 'curl -F ' "bid={} ".format(amountmsat) + '-F ' + ' "message=' + message + sentby + '" ' + url
    sh = os.popen(curl)
    clear()
    blogo()
    sh0 = sh.read()
    while True:
        if 'Bid too low' in sh0:
            print("\n\t\033[1;31;40mATENTION: Per byte bid cannot be below 50 millisatoshis per byte.\033[0;37;40m\n")
            print("Try again...\n")
            message = input("\nInsert your Message: ")
            a = sys.getsizeof(message)
            mini = 6000
            n = (a * 50)
            if n <= mini:
                q = str(mini)
            elif n >= mini:
                k = n + mini
                q = str(k)

            url = 'https://api.blockstream.space/order'
            sentby = " - PyBLOCK."
            print("\n\tATENTION: YOU NEED TO PAY \033[1;31;40m" + q + "\033[0;37;40m MilliSats")
            amountmsat = input("\nInsert the amount in MSats: ")
            curl = 'curl -F ' "bid={} ".format(amountmsat) + '-F ' + ' "message=' + message + sentby + '" ' + url
            sh = os.popen(curl)
            clear()
            blogo()
            sh0 = sh.read()
        elif 'lightning_invoice' in sh0:
            break

    sh1 = str(sh0)
    shh = sh1.split(',')
    invoice = str(shh[6])

    #---------------Token-----------
    authtoken = str(shh[0])
    authtoken1 = authtoken.split(':')
    token = authtoken1[1]
    #---------------End Token-------

    #---------------Order-----------
    uuid = str(shh[1])
    uuid1 = uuid.split(':')
    order = uuid1[1]
    #---------------End Order-------

    #---------------Amount----------
    msat = str(shh[3])
    msat1 = msat.split(':')
    amount = msat1[1]
    #---------------End Amount------

    orderid = str(shh[1])
    ln1 = invoice.split(':')
    ln2 = str(ln1[1])
    cln = ln2.strip('"')
    print("\n\033[0;37;40mYour Token Authorization: \033[1;31;40m" + token + "\033[0;37;40m")
    print("\033[0;37;40mYour Order Number: \033[1;31;40m" + order + "\033[0;37;40m")
    print("\033[0;37;40mAmount in MSats: \033[1;33;40m" + amount + "\033[0;37;40m\n")
    clear()
    blogo()
    node_not = input("Do you want to pay this message with your node? Y/n: ")
    if node_not in ["Y", "y"]:
        lndconnectload = {"ip_port":"", "tls":"", "macaroon":"", "ln":""}
        lndconnectData = pickle.load(open("blndconnect.conf", "rb")) # Load the file 'bclock.conf'
        lndconnectload = lndconnectData # Copy the variable pathv to 'path'
        if lndconnectload['ip_port']:
            print("\nInvoice: " + cln + "\n")
            payinvoice()
        elif lndconnectload['ln']:
            print("\nInvoice: " + cln + "\n")
            localpayinvoice()
    elif node_not in ["N", "n"]:
        print("\033[1;30;47m")
        qr.add_data(cln)
        qr.print_ascii()
        print("\033[0;37;40m")
        print("\nLND Invoice: " + cln + "\n")
        sh.close()
        continue1 = input("Continue? Y: ")
        if continue1 == "Y" or continue1 == "y":
            donate()
        else:
            t.sleep(2)

def apisenderFile():
    qr = qrcode.QRCode(
    version=1,
    error_correction=qrcode.constants.ERROR_CORRECT_L,
    box_size=10,
    border=4,
    )
    url = 'https://api.blockstream.space/order'
    message = input("\nInsert the path to the File: ")
    print("ATENTION: Minimum amount for sending a File is 50000 MSats")
    amountmsat = input("\nInsert the amount in MSats: ")
    curl = 'curl -F ' "bid={} ".format(amountmsat) + '-F ' + ' "file=@' + message + '" ' + url
    sh = os.popen(curl)
    sh0 = sh.read()
    while True:
        try:
            if 'Bid too low' in sh0:
                print("\n\t\033[1;31;40mATENTION: Per byte bid cannot be below 50 millisatoshis per byte.\033[0;37;40m\n")
                print("Try again...\n")
                url = 'https://api.blockstream.space/order'
                message = input("\nInsert the path to the File: ")
                print("ATENTION: Minimum amount for sending a File is 50000 MSats")
                amountmsat = input("\nInsert the amount in MSats: ")
                curl = 'curl -F ' "bid={} ".format(amountmsat) + '-F ' + ' "file=@' + message + '" ' + url
                sh = os.popen(curl)
                sh0 = sh.read()
            elif 'lightning_invoice' in sh0:
                break
        except:
            break

    sh1 = str(sh0)
    shh = sh1.split(',')
    invoice = str(shh[6])

    #---------------Token-----------
    authtoken = str(shh[0])
    authtoken1 = authtoken.split(':')
    token = authtoken1[1]
    #---------------End Token-------

    #---------------Order-----------
    uuid = str(shh[1])
    uuid1 = uuid.split(':')
    order = uuid1[1]
    #---------------End Order-------

    #---------------Amount----------
    msat = str(shh[3])
    msat1 = msat.split(':')
    amount = msat1[1]
    #---------------End Amount------

    orderid = str(shh[1])
    ln1 = invoice.split(':')
    ln2 = str(ln1[1])
    cln = ln2.strip('"')
    print("\n\033[0;37;40mYour Token Authorization: \033[1;31;40m" + token + "\033[0;37;40m")
    print("\033[0;37;40mYour Order Number: \033[1;31;40m" + order + "\033[0;37;40m")
    print("\033[0;37;40mAmount in MSats: \033[1;33;40m" + amount + "\033[0;37;40m")
    clear()
    blogo()
    try:
        node_not = input("Do you want to pay this message with your node? Y/n: ")
        if node_not in ["Y", "y"]:
            lndconnectload = {"ip_port":"", "tls":"", "macaroon":"", "ln":""}
            lndconnectData = pickle.load(open("blndconnect.conf", "rb")) # Load the file 'bclock.conf'
            lndconnectload = lndconnectData # Copy the variable pathv to 'path'
            if lndconnectload['ip_port']:
                print("\nInvoice: " + cln + "\n")
                payinvoice()
            elif lndconnectload['ln']:
                print("\nInvoice: " + cln + "\n")
                localpayinvoice()
        elif node_not in ["N", "n"]:
            print("\033[1;30;47m")
            qr.add_data(cln)
            qr.print_ascii()
            print("\033[0;37;40m")
            print("\nLND Invoice: " + cln)
            sh.close()
            continue1 = input("Continue? Y: ")
            if continue1 == "Y" or continue1 == "y":
                donate()
            else:
                t.sleep(2)
    except:
        pass

def devAddr():
    qr = qrcode.QRCode(
    version=1,
    error_correction=qrcode.constants.ERROR_CORRECT_L,
    box_size=10,
    border=4,
    )
    print("\n\t\t\033[1;33;44mGive us some love and \033[1;31;44mDONATE\033[1;33;44m us! We will appreciate it. This will be a boost to continue this beautiful project! \033[0;37;40m")
    url = 'https://api.tippin.me/v1/public/addinvoice/royalfield370'
    response = requests.get(url)
    responseB = str(response.text)
    responseC = responseB
    lnreq = responseC.split(',')
    lnbc1 = lnreq[1]
    lnbc1S = str(lnbc1)
    lnbc1R = lnbc1S.split(':')
    lnbc1W = lnbc1R[1]
    ln = str(lnbc1W)
    ln1 = ln.strip('"')
    clear()
    blogo()
    try:
        node_not = input("Do you want to pay this tip with your node? Y/n: ")
        if node_not in ["Y", "y"]:
            lndconnectload = {"ip_port":"", "tls":"", "macaroon":"", "ln":""}
            lndconnectData = pickle.load(open("blndconnect.conf", "rb")) # Load the file 'bclock.conf'
            lndconnectload = lndconnectData # Copy the variable pathv to 'path'
            if lndconnectload['ip_port']:
                print("\nInvoice: " + ln1 + "\n")
                payinvoice()
            elif lndconnectload['ln']:
                print("\nInvoice: " + ln1 + "\n")
                localpayinvoice()
        elif node_not in ["N", "n"]:
            print("\033[1;30;47m")
            qr.add_data(ln1)
            qr.print_ascii()
            print("\033[0;37;40m")
            print("LND Invoice: " + ln1)
            response.close()
    except:
        pass

def donate():
    print("""\t\t
    \033[1;31;40mPyBLOCK\033[0;37;40m Menu

    \033[1;32;40mA.\033[0;37;40m Donate
    <<< Back Control + C
    \n\n""")
    menuB = (input("\033[1;32;40mSelect option: \033[0;37;40m"))
    if menuB in ["A", "a"]:
        devAddr()



#Developer: Curly60e
#Tester: __B__T__C__
#ℙ𝕪𝔹𝕃𝕆ℂ𝕂 𝕚𝕥𝕤 𝕒 𝔹𝕚𝕥𝕔𝕠𝕚𝕟 𝔻𝕒𝕤𝕙𝕓𝕠𝕒𝕣𝕕 𝕨𝕚𝕥𝕙 ℂ𝕪𝕡𝕙𝕖𝕣𝕡𝕦𝕟𝕜 𝕒𝕖𝕤𝕥𝕙𝕖𝕥𝕚𝕔.


import requests
import qrcode
import pickle
from nodeconnection import *

def donationAddr():
    qr = qrcode.QRCode(
    version=1,
    error_correction=qrcode.constants.ERROR_CORRECT_L,
    box_size=10,
    border=4,
    )
    url = 'bc1qjzaz34nv2ev55vfdu9m5qh0zq0fwcn6c7pkcrv'
    print("\033[1;30;47m")
    qr.add_data(url)
    qr.print_ascii()
    print("\033[0;37;40m")
    qr.clear()
    print(f"Bitcoin Address Bech32: {url}")

def donationPayNym():
    qr = qrcode.QRCode(
    version=1,
    error_correction=qrcode.constants.ERROR_CORRECT_L,
    box_size=10,
    border=4,
    )
    url = 'PM8TJhNTTq3YVocXuPtLjKx7pKkdUxqwTerWJ2j2a7dNitgyMmBPN6gK61yE17N2vgvQvKYokXktt6D6GZFTmocvDJhaUJfHt7ehEMmthjsT3NQHseFM'
    print("\033[1;30;47m")
    qr.add_data(url)
    qr.print_ascii()
    print("\033[0;37;40m")
    qr.clear()
    print(f"PayNym: {url}")

#Dev LN
def donationLN():
    qr = qrcode.QRCode(
    version=1,
    error_correction=qrcode.constants.ERROR_CORRECT_L,
    box_size=10,
    border=4,
    )
    amt = input("Amount: ")
    curl = (
        'curl -X POST https://lnbits.com/api/v1/payments -d '
        + "'{"
        + f""""out": false, "amount": {amt}, "memo": "Donation" """
        + "}'"
        + """ -H "X-Api-Key: 1d646820055e4e2da218e801eaacfc94 " -H "Content-type: application/json" """
    )

    sh = os.popen(curl).read()
    clear()
    blogo()
    n = str(sh)
    d = json.loads(n)
    q = d['payment_request']
    c = q.lower()
    while True:
        print("\033[1;30;47m")
        qr.add_data(c)
        qr.print_ascii()
        print("\033[0;37;40m")
        qr.clear()
        print(f"Lightning Invoice: {c}")
        dn = str(d['checking_id'])
        t.sleep(10)
        checkcurl = (
            f'curl -X GET https://lnbits.com/api/v1/payments/{dn}'
            + """ -H "X-Api-Key: 1d646820055e4e2da218e801eaacfc94" -H "Content-type: application/json" """
        )

        rsh = os.popen(checkcurl).read()
        clear()
        blogo()
        nn = str(rsh)
        dd = json.loads(nn)
        db = dd['paid']
        if db is not True:
            continue

        clear()
        blogo()
        tick()
        bitLN['pd'] = "PAID"

#Tester Address
def donationAddrTst():
    qr = qrcode.QRCode(
    version=1,
    error_correction=qrcode.constants.ERROR_CORRECT_L,
    box_size=10,
    border=4,
    )
    url = 'bc1qwtzwu2evtchkvnf3ey6520yprsyv7vrjvhula5'
    print("\033[1;30;47m")
    qr.add_data(url)
    qr.print_ascii()
    print("\033[0;37;40m")
    qr.clear()
    print(f"Bitcoin Address Bech32: {url}")

#Tester LN
def donationLNTst():
    qr = qrcode.QRCode(
    version=1,
    error_correction=qrcode.constants.ERROR_CORRECT_L,
    box_size=10,
    border=4,
    )
    url = 'https://api.tippin.me/v1/public/addinvoice/__B__T__C__'
    response = requests.get(url)
    responseB = str(response.text)
    responseC = responseB
    lnreq = responseC.split(',')
    lnurl = lnreq[1]
    lnurlS = str(lnbc1)
    node_not = input("Are you Satoshi? Y/n: ")
    if node_not in ["Y", "y", "N", "n"]:
        print("\033[1;30;47m")
        lnurlR = lnurlS.split(':')
        lnurlW = lnurlR[1]
        ln = str(lnurlW)
        ln1 = ln.strip('"')
        qr.add_data(ln1)
        qr.print_ascii()
        print("\033[0;37;40m")
        print(f"LNURL: {ln1}")
        qr.clear()
        response.close()

def decodeQR():
    qr = qrcode.QRCode(
    version=1,
    error_correction=qrcode.constants.ERROR_CORRECT_L,
    box_size=10,
    border=4,
    )
    url = input("Insert your Bitcoin Address to show the QRCode: ")
    print("\033[1;30;47m")
    qr.add_data(url)
    qr.print_ascii()
    print("\033[0;37;40m")
    qr.clear()
    print(f"Bitcoin Address: {url}")



#Developer: Curly60e
#PyBLOCK its a clock of the Bitcoin blockchain.


import os
import os.path
import time as t


def readFile():
    try:
        print ("\n\033[1;34;40mWaiting for new data...\n")
        downloadsFolder = 'downloads/'
        while True:
            if not os.listdir(downloadsFolder):
                continue
            else:
                print("\t\t\n\033[1;33;40mNew message from Space just arrived...\033[0;37;40m\n")
                os.system("cat downloads/*")
                os.system("rm downloads/*")

    except:
        os.system("ps -ef | grep api_data_reader.py | grep -v grep | awk '{print $2}' | xargs kill -9")
        os.system("ps -ef | grep demo-rx.py | grep -v grep | awk '{print $2}' | xargs kill -9")
